import { reactive } from '@vue/reactivity';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var defaultOptions = {
    clickTime: 3,
    interval: 500
};
/**
 * listen multiple click
 * @param el        target element
 * @param callback  multiple click callback
 * @param options   options
 * @returns         a function to clear side effect
 */
function useMultipleClick(el, callback, options) {
    var opt = __assign(__assign({}, defaultOptions), options);
    var clicks = [];
    function clickHandler(e) {
        var curTime = performance.now();
        clicks.push({
            nativeEvent: e,
            time: curTime
        });
        if (clicks.length === opt.clickTime) {
            if (curTime - clicks[0].time <= opt.interval) {
                callback.call(el, { clicks: clicks.concat([]) });
                clicks = [];
            }
            else {
                clicks.shift();
            }
        }
    }
    el.addEventListener('click', clickHandler);
    return function () {
        el.removeEventListener('click', clickHandler);
    };
}

/**
 * use mouse position
 * @return pos reactive mouse position
 * @return clear: clear event listener
 */
function useMousePos() {
    var pos = reactive({
        x: 0,
        y: 0
    });
    var onMouseMoveHandler = function (e) {
        pos.x = e.clientX;
        pos.y = e.clientY;
    };
    document.addEventListener('mousemove', onMouseMoveHandler);
    var clear = function () {
        document.removeEventListener('mousemove', onMouseMoveHandler);
    };
    return {
        pos: pos,
        clear: clear
    };
}

export { useMousePos, useMultipleClick };
